/// <reference types="node" />
import { EventEmitter } from 'events';
export declare class DashboardMessageBus extends EventEmitter {
    publishPort: number;
    subscribePort: number;
    host: string;
    private publishServer;
    private subscribeServer;
    private publishers;
    private subscribers;
    private unfulfilledRequests;
    constructor(publishPort: number, subscribePort: number, host?: string);
    /**
     * Start the DashboardMessageBus
     * @dev This starts separate websocket servers for subscribers/publishers
     */
    start(): Promise<void>;
    /**
     * Wait for the message bus to be "ready" to process requests (i.e. having any subscribers).
     * @dev Polls every second to see if the number of subscribers > 0
     */
    ready(): Promise<void>;
    /**
     * Close both websocket servers
     * @dev Emits a "terminate" event
     */
    terminate(): Promise<void>;
    /**
     * Process a message `data` coming from `publisher` by sending it to `subscribers`
     * and return the first received response to the `publisher`
     */
    private processRequest;
    private invalidateMessage;
    private logToPublishers;
    private logToSubscribers;
    private logToAll;
    private logTo;
    /**
     * Add a publisher so it can be used to send requests to
     * @dev Also sends all backlogged (unfulfilled) requests upon connection
     */
    private addSubscriber;
    /**
     * Remove a subscriber
     * @dev Will cause the server to terminate if this was the last connection
     */
    private removeSubscriber;
    /**
     * Add a publisher and set up message listeners to process their requests
     */
    private addPublisher;
    /**
     * Remove a publisher and their corresponding requests
     * @dev Will cause the server to terminate if this was the last connection
     */
    private removePublisher;
    private terminateIfNoConnections;
    private clearRequestsForPublisher;
}
