"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DashboardProvider = void 0;
const util_1 = require("util");
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const delay_1 = __importDefault(require("delay"));
const open_1 = __importDefault(require("open"));
const msgbus_1 = require("@securerpc/msgbus");
const dashboard_1 = require("@securerpc/dashboard");
const promise_timeout_1 = require("promise-timeout");
const debug_1 = __importDefault(require("debug"));
class DashboardProvider {
    constructor(options = {}) {
        var _a, _b, _c, _d, _e, _f;
        this.concurrentRequests = 0;
        this.connecting = false;
        this.dashboardHost = (_a = options.dashboardHost) !== null && _a !== void 0 ? _a : 'localhost';
        this.dashboardPort = (_b = options.dashboardPort) !== null && _b !== void 0 ? _b : 24012;
        this.timeoutSeconds = (_c = options.timeoutSeconds) !== null && _c !== void 0 ? _c : 120;
        this.keepAlive = (_d = options.keepAlive) !== null && _d !== void 0 ? _d : false;
        this.verbose = (_e = options.verbose) !== null && _e !== void 0 ? _e : false;
        this.autoOpen = (_f = options.autoOpen) !== null && _f !== void 0 ? _f : true;
        // Start a dashboard at the provided port (will silently fail if the dashboard address is already in use)
        const dashboardOptions = {
            port: this.dashboardPort,
            host: this.dashboardHost,
            rpc: false,
            verbose: this.verbose,
            autoOpen: false,
        };
        (0, dashboard_1.startDashboardInBackground)(dashboardOptions);
        if (this.autoOpen) {
            (0, open_1.default)(`http://${this.dashboardHost}:${this.dashboardPort}`);
        }
    }
    send(payload, callback) {
        const sendInternal = (payload) => this.sendInternal(payload);
        (0, util_1.callbackify)(sendInternal)(payload, callback);
    }
    sendAsync(payload, callback) {
        this.send(payload, callback);
    }
    terminate() {
        var _a;
        (_a = this.socket) === null || _a === void 0 ? void 0 : _a.terminate();
    }
    sendInternal(payload) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready();
            const message = (0, msgbus_1.createMessage)('provider', payload);
            this.concurrentRequests += 1;
            const { payload: response } = yield (0, promise_timeout_1.timeout)((0, msgbus_1.sendAndAwait)(this.socket, message), this.timeoutSeconds * 1000);
            this.concurrentRequests -= 1;
            if (this.shouldTerminate()) {
                this.terminate();
            }
            if (response.error) {
                throw response.error;
            }
            return response;
        });
    }
    shouldTerminate() {
        if (this.keepAlive)
            return false;
        if (this.concurrentRequests !== 0)
            return false;
        return true;
    }
    ready() {
        return __awaiter(this, void 0, void 0, function* () {
            // Don't create a new connection to the message bus while we're already connecting
            if (this.connecting) {
                yield (0, delay_1.default)(1000);
                return this.ready();
            }
            if (this.socket && this.socket.readyState === isomorphic_ws_1.default.OPEN) {
                this.connecting = false;
                return;
            }
            this.connecting = true;
            try {
                const { publishPort } = yield (0, msgbus_1.getMessageBusPorts)(this.dashboardPort, this.dashboardHost);
                this.socket = yield (0, msgbus_1.connectToMessageBusWithRetries)(publishPort, this.dashboardHost);
                if (this.verbose)
                    this.setupLogging();
            }
            finally {
                this.connecting = false;
            }
        });
    }
    setupLogging() {
        var _a;
        (_a = this.socket) === null || _a === void 0 ? void 0 : _a.addEventListener('message', (event) => {
            if (typeof event.data !== 'string') {
                event.data = event.data.toString();
            }
            const message = (0, msgbus_1.base64ToJson)(event.data);
            if (message.type === 'log') {
                const logMessage = message;
                const debug = (0, debug_1.default)(logMessage.payload.namespace);
                debug.enabled = true;
                debug(logMessage.payload.message);
            }
        });
    }
}
exports.DashboardProvider = DashboardProvider;
//# sourceMappingURL=DashboardProvider.js.map