import WebSocket, { ServerOptions } from 'isomorphic-ws';
import type { Message, PortsConfig } from './types';
/**
 * Convert any JS object or value to a base64 representation of it
 */
export declare const jsonToBase64: (json: any) => string;
/**
 * Convert the base64 representation of a JS object or value to its JS representation
 * @dev This is the reverse of `jsonToBase64` and is not expected to work with other base64 formats
 */
export declare const base64ToJson: (base64: string) => any;
/**
 * Starts a websocket server and waits for it to be opened
 * @dev If you need to attach event listeners *before* the server connection opens,
 * do not use this function since it resolves *after* the connection is opened
 */
export declare const startWebSocketServer: (options: ServerOptions) => Promise<WebSocket.Server>;
export declare const createMessage: (type: string, payload: any) => Message;
/**
 * Broadcast a message to multiple websocket connections and disregard them
 */
export declare const broadcastAndDisregard: (sockets: WebSocket[], message: Message) => void;
/**
 * Broadcast a message to multuple websocket connections and return the first response
 */
export declare const broadcastAndAwaitFirst: (sockets: WebSocket[], message: Message) => Promise<any>;
/**
 * Send a message to a websocket connection and await a matching response
 * @dev Responses are matched by looking at received messages that match the ID of the sent message
 */
export declare const sendAndAwait: (socket: WebSocket, message: Message) => Promise<any>;
export declare const connectToMessageBusWithRetries: (port: number, host?: string, retries?: number) => Promise<WebSocket>;
export declare const connectToMessageBus: (port: number, host?: string) => Promise<WebSocket>;
export declare const getMessageBusPorts: (dashboardPort: number, dashboardHost?: string, retries?: number) => Promise<PortsConfig>;
